"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeBundlingOptions = exports.prepareBuild = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const child_process_1 = require("child_process");
const cdk = __importStar(require("@aws-cdk/core"));
const uuid_1 = require("uuid");
const temp_dir_1 = __importDefault(require("temp-dir"));
const walkdir_1 = __importDefault(require("walkdir"));
const metaDataPath = path_1.default.normalize(path_1.default.join(__dirname, "..", ".metadata.json"));
const scriptsPath = path_1.default.normalize(path_1.default.join(__dirname, "..", "scripts"));
const runtimeCache = {};
let metadata;
const fetchMetaData = () => {
    if (metadata == null) {
        if (fs_1.default.existsSync(metaDataPath)) {
            metadata = JSON.parse(fs_1.default.readFileSync(metaDataPath).toString("utf-8"));
        }
        else {
            metadata = {
                hashes: {}
            };
        }
    }
};
const saveMetaData = () => {
    fs_1.default.writeFileSync(metaDataPath, JSON.stringify(metadata, null, 2));
};
const makeBuildDirectory = () => {
    const dir = path_1.default.join(temp_dir_1.default, uuid_1.v4(), "build");
    fs_1.default.mkdirSync(dir, { recursive: true });
    return dir;
};
const makeDockerFile = (runtime, useNpm7 = false) => (`
FROM ${runtime.bundlingImage.image}:latest

COPY writeDependencies.js /lambda.js/writeDependencies.js

${useNpm7 ? "RUN npm i -g npm@^7" : ""}
RUN npm i -g typescript
`);
const runScriptPrepareBuild = (event) => {
    const eventEncoded = Buffer.from(JSON.stringify(event)).toString("base64");
    const output = child_process_1.execSync(`"${process.execPath}" "${path_1.default.join(scriptsPath, "prepareBuild.js")}" ${eventEncoded}
  `);
    try {
        return JSON.parse(Buffer.from(output.toString("utf-8"), "base64").toString("utf-8"));
    }
    catch (e) {
        console.log("Output: ", output.toString("utf-8"));
        throw new Error("Could not parse result from prepareBuild script!");
    }
};
function prepareBuild(options) {
    const temporaryDirectory = makeBuildDirectory();
    const dockerfile = makeDockerFile(options.runtime, options._bundleUsingNpm7);
    let cache;
    if (runtimeCache[options.directory] != null) {
        cache = runtimeCache[options.directory];
    }
    const scriptResult = runScriptPrepareBuild({
        entry: options.entry,
        sourceDirectory: options.directory,
        buildDirectory: temporaryDirectory,
        files: {
            dockerfile,
            script: fs_1.default.readFileSync(path_1.default.normalize(path_1.default.join(__dirname, "..", "scripts", "writeDependencies.js"))).toString("utf-8")
        },
        cache: {
            depcheck: cache === null || cache === void 0 ? void 0 : cache.depcheck
        },
    });
    if (cache == null) {
        runtimeCache[options.directory] = {
            ...scriptResult.dependencyScan.cache,
            paths: walkdir_1.default.sync(options.directory, {
                return_object: true,
                filter: (_, files) => files.filter(it => it != "node_modules")
            })
        };
        cache = runtimeCache[options.directory];
    }
    return {
        hasDependencies: scriptResult.hasDependencies,
        dependencies: {
            packageJson: scriptResult === null || scriptResult === void 0 ? void 0 : scriptResult.dependencyScan.packageJson,
            packageLock: scriptResult === null || scriptResult === void 0 ? void 0 : scriptResult.dependencyScan.packageLock,
        },
        build: {
            directory: temporaryDirectory,
            hash: scriptResult.buildDirHash
        },
        info: {
            sources: scriptResult === null || scriptResult === void 0 ? void 0 : scriptResult.dependencyScan.sources
        },
        cache
    };
}
exports.prepareBuild = prepareBuild;
function makeBundlingOptions(options, commands = []) {
    fetchMetaData();
    let dockerImage;
    if (metadata.hashes[options.build.hash] != null) {
        dockerImage = cdk.DockerImage.fromRegistry(metadata.hashes[options.build.hash].dockerImage);
    }
    else {
        dockerImage = cdk.DockerImage.fromBuild(options.build.directory);
        metadata.hashes[options.build.hash] = {
            dockerImage: dockerImage.image
        };
    }
    saveMetaData();
    return {
        image: dockerImage,
        command: commands.length > 0 ? ["bash", "-c", commands.join(" && ")] : undefined,
        environment: {
            HOME: "/tmp/home",
            LAMBDAJS_AGENT_INPUT: Buffer.from(JSON.stringify(options.dependencies)).toString("base64"),
            LAMBDAJS_AGENT_OUTPUT: "/asset-output/nodejs"
        }
    };
}
exports.makeBundlingOptions = makeBundlingOptions;
//# sourceMappingURL=build.js.map