"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Function = void 0;
const path_1 = __importDefault(require("path"));
const cdk = __importStar(require("@aws-cdk/core"));
const lambda = __importStar(require("@aws-cdk/aws-lambda"));
const build_1 = require("./build");
const makeLambdaExcludes = (directory, sources, paths) => {
    const include = [
        ...sources,
        path_1.default.join(directory, "package.json")
    ];
    const files = Object.entries(paths);
    let exclude = [];
    for (const [name, entry] of files) {
        if (entry.isFile()) {
            if (!include.includes(name)) {
                exclude.push(name);
            }
        }
        else {
            const containsNothing = files
                .filter(([it]) => it.startsWith(name))
                .every(([it]) => !include.includes(it));
            if (containsNothing) {
                exclude.push(name);
            }
        }
    }
    exclude = exclude
        .map(it => path_1.default.relative(directory, it))
        .map(it => it.replace(/\\/g, '/'));
    return [
        ...exclude,
        "node_modules"
    ];
};
class Function extends lambda.Function {
    constructor(scope, id, props) {
        props.entry = path_1.default.normalize(props.entry);
        const { entry, function: func, directory = path_1.default.dirname(entry), inplace = false, runtime = lambda.Runtime.NODEJS_14_X, includePaths, ...functionProps } = props;
        const preparation = build_1.prepareBuild({ runtime, entry, directory, inplace, _bundleUsingNpm7: props._bundleUsingNpm7 });
        let includes = [...preparation.info.sources];
        if (includePaths != null) {
            includes = [
                ...includes,
                ...includePaths
            ];
        }
        let codeOptions = {
            exclude: makeLambdaExcludes(directory, includes, preparation.cache.paths)
        };
        if (entry.endsWith(".ts") || entry.endsWith(".ts/")) {
            codeOptions = {
                ...codeOptions,
                bundling: build_1.makeBundlingOptions({ build: preparation.build, dependencies: preparation.dependencies }, [
                    "cp -R /asset-input/. /asset-output",
                    "cd /asset-output",
                    "tsc *.ts"
                ])
            };
        }
        const lambdaFilePath = path_1.default.join(path_1.default.relative(directory, path_1.default.dirname(entry)), path_1.default.parse(entry).name).replace(/\\/g, '/');
        const handler = `${lambdaFilePath}.${func}`;
        super(scope, id, {
            runtime,
            handler,
            code: lambda.Code.fromAsset(directory, codeOptions),
            ...functionProps,
        });
        if (preparation.hasDependencies) {
            this.layer = new lambda.LayerVersion(scope, `${id}Layer`, {
                code: lambda.Code.fromAsset(directory, {
                    exclude: ["*", "!package.json", "!package-lock.json"],
                    assetHashType: cdk.AssetHashType.SOURCE,
                    bundling: build_1.makeBundlingOptions({ build: preparation.build, dependencies: preparation.dependencies }, [
                        "mkdir /asset-output/nodejs",
                        "node /lambda.js/writeDependencies.js",
                        "cd /asset-output/nodejs",
                        "npm ci --production > /dev/null"
                    ])
                })
            });
            this.addLayers(this.layer);
        }
    }
}
exports.Function = Function;
//# sourceMappingURL=lambda.js.map